#pragma once

#include <windows.h>
#include <string.h>
#include "vfm.h"
#include "avisynth.h"

extern "C"
{
#include "liba52\a52.h"
#include "liba52\a52_internal.h"
//#include "mm_accel.h"
}

class m2AudioAC3Source : public IClip
{
public:
	static AVSValue __cdecl Create(AVSValue args, void *, IScriptEnvironment *env);

	// Initialization
	m2AudioAC3Source(const char *FileName, int Downmix, int DRC, IScriptEnvironment *env);
	~m2AudioAC3Source();

	// IClip interface
	const VideoInfo& __stdcall GetVideoInfo();
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	bool __stdcall GetParity(int n);
	void __stdcall GetAudio(void *buf, __int64 start, __int64 count, IScriptEnvironment *env);
	void __stdcall SetCacheHints(int cachehints, int frame_range);

private:
	// General information
	VideoInfo Info;						// required by AviSynth

	// Stream
	VFILE   *Stream;					// stream handle
	char    *StreamName;				// name of the stream
	__int64  StreamOffset;				// offset to start of the audio data
	__int64  StreamLength;				// total length of the stream

	// Audio information
	int     ChannelCount;				// number of channels
	__int64 SampleCount;				// number of samples per channel in the stream
	int     FrameCount;					// number of entire frames in the stream
	int     Flags, Samplerate, Bitrate;	// bit stream information
	bool    LFE;						// is the LFE channel present?
	int     DecFlags;					// flags used for decoding
	float   Level;						// level of output samples
	int     Accel;						// decoder acceleration
	int		_drc;						// DRC

	// Buffers
	a52_state_t   *State;				// decoder state structure
	unsigned char *Frame;				// raw buffer
	int FrameIndex, FrameLength;		// index and length of the current frame

	float   *Buffer;					// samples from the current frame in interleaved order (framebuffer)
	int      BufferSize;				// size of the framebuffer
	__int64  BufferStart, BufferEnd;	// sample range of the framebuffer

	// Operations
	int Synchronize(int& Length, int& Flags, int& Samplerate, int& Bitrate);
	bool ReadFrame();
	bool DecodeFrame();
	void EmptyFrame();
};

extern "C"
{
#include "libdts\dts.h"
#include "libdts\dts_internal.h"
//#include "mm_accel.h"
}

/*-----------------------------------------------------------------------------
	Constants
-----------------------------------------------------------------------------*/

#define SAMPLES_PER_BLOCK	256			// number of samples per channel in a block

#define MAX_DETECT_ERROR	1024		// max detection error (in bytes)

/*-----------------------------------------------------------------------------
	m2AudioDTSSource: Input filter class
-----------------------------------------------------------------------------*/

class m2AudioDTSSource : public IClip
{
public:
	static AVSValue __cdecl Create(AVSValue args, void *, IScriptEnvironment *env);

	// Initialization
	m2AudioDTSSource(const char *FileName, int Downmix, int DRC, IScriptEnvironment *env);
	~m2AudioDTSSource();

	// IClip interface
	const VideoInfo& __stdcall GetVideoInfo();
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	bool __stdcall GetParity(int n);
	void __stdcall GetAudio(void *buf, __int64 start, __int64 count, IScriptEnvironment *env);
	void __stdcall SetCacheHints(int cachehints, int frame_range);

private:
	// General information
	VideoInfo Info;						// required by AviSynth

	// Stream
	VFILE   *Stream;					// stream handle
	char    *StreamName;				// name of the stream
	__int64  StreamOffset;				// offset to start of the audio data
	__int64  StreamLength;				// total length of the stream

	// Audio information
	int     ChannelCount;				// number of channels
	__int64 SampleCount;				// number of samples per channel in the stream
	int     FrameCount;					// number of entire frames in the stream
	int     BlocksPerFrame;				// number of blocks in a frame
	int     SamplesPerFrame;			// number of samples per channel in a frame
	int     Flags, Samplerate, Bitrate;	// bit stream information
	bool    LFE;						// is the LFE channel present?
	int     DecFlags;					// flags used for decoding
	float   Level;						// level of output samples
	int     Accel;						// decoder acceleration
	int		_drc;						// DRC

	// Buffers
	dts_state_t   *State;				// decoder state structure
	unsigned char *Frame;				// raw buffer
	int FrameIndex, FrameLength;		// index and length of the current frame

	float   *Buffer;					// samples from the current frame in interleaved order (framebuffer)
	int      BufferSize;				// size of the framebuffer
	__int64  BufferStart, BufferEnd;	// sample range of the framebuffer

	// Operations
	int  Synchronize(int& Length, int& Flags, int& Samplerate, int& Bitrate);
	bool ReadFrame();
	bool DecodeFrame();
	void EmptyFrame();
};

class m2AudioLPCMSource : public IClip
{
public:
	static AVSValue __cdecl Create(AVSValue args, void *, IScriptEnvironment *env);

	// Initialization
	m2AudioLPCMSource(const char *FileName, int SampleRate, int SampleBits, int Channels, IScriptEnvironment *env);
	~m2AudioLPCMSource();

	// IClip interface
	const VideoInfo& __stdcall GetVideoInfo();
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	bool __stdcall GetParity(int n);
	void __stdcall GetAudio(void *buf, __int64 start, __int64 count, IScriptEnvironment *env);
	void __stdcall SetCacheHints(int cachehints, int frame_range);

private:
	// General information
	VideoInfo Info;									// required by AviSynth

	// Stream
	VFILE  *Stream;									// stream handle
	char   *StreamName;								// name of the stream
	__int64 StreamLength;							// total length of the stream

	// Audio information
	int	    Quant, InQuant;							// sample quantization (number of bits per sample)
	int     ChannelCount, InChannelCount;			// number of channels
	__int64 SampleCount;							// number of samples per channel in the stream
	int     SamplesPerFrame;						// number of samples per channel in a frame

	// Buffers
	unsigned char *Frame;							// raw buffer
	int            FrameIndex, FrameLength;			// index and length of the current frame

	unsigned char *Buffer,							// processed samples from the current frame (pointer to the real framebuffer)
				  *AuxBuffer1, *AuxBuffer2;			
	int			   AuxBuffer1Size, AuxBuffer2Size;	// size of the auxiliary framebuffers
	__int64        BufferStart, BufferEnd;			// sample range of the framebuffer

	bool		   bIsWavFile;

	// Operations
	bool ReadFrame();
};

extern "C"
{
#include "libmad\mad.h"
}

/*-----------------------------------------------------------------------------
	Constants
-----------------------------------------------------------------------------*/


#define SAMPLES_PER_FRAME_L1	384			// number of samples per channel in a Layer 1 frame
#define SAMPLES_PER_FRAME_L2_L3	1152		// number of samples per channel in a Layer 2/3 frame

#define MAX_DETECT_ERROR		1024		// max detection error (in bytes)

/*-----------------------------------------------------------------------------
	m2AudioMPASource: Input filter class
-----------------------------------------------------------------------------*/

class m2AudioMPASource : public IClip
{
public:
	static AVSValue __cdecl Create(AVSValue args, void *, IScriptEnvironment *env);

	// Initialization
	m2AudioMPASource(const char *FileName, int Downmix, IScriptEnvironment *env);
	~m2AudioMPASource();

	// IClip interface
	const VideoInfo& __stdcall GetVideoInfo();
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	bool __stdcall GetParity(int n);
	void __stdcall GetAudio(void *buf, __int64 start, __int64 count, IScriptEnvironment *env);
	void __stdcall SetCacheHints(int cachehints, int frame_range);

private:
	// General information
	VideoInfo Info;						// required by AviSynth

	// Stream
	VFILE   *Stream;					// stream handle
	char    *StreamName;				// name of the stream
	__int64  StreamOffset;				// offset to start of the audio data
	__int64  StreamLength;				// total length of the stream

	// Audio information
	int     ChannelCount;				// number of channels
	__int64 SampleCount;				// number of samples per channel in the stream
	int     FrameCount;					// number of entire frames in the stream
	int     SamplesPerFrame;			// number of samples per channel in a frame
	int     Layer, Samplerate, Bitrate;	// bit stream information
	double  AvgFrameLength;				// average frame length (required for seeking)
	int     Padding;					// padding (in bytes)

	// MAD stuff
	mad_stream MADStream;
	mad_frame  MADFrame;
	mad_synth  MADSynth;

	// Buffers
	unsigned char *Frame;				// raw buffer
	int FrameIndex, FrameLength;		// index and length of the current frame

	float  *Buffer;						// samples from the current frame in interleaved order (framebuffer)
	int     BufferSize;					// size of the framebuffer
	__int64 BufferStart, BufferEnd;		// sample range of the framebuffer

	// Operations
	int  Synchronize(int& Length, int& Layer, int& Channels, int& Samplerate, int& Bitrate);
	bool ReadFrame();
	bool DecodeFrame();
	void EmptyFrame();

	// Utilities
	int SyncInfo(unsigned char *Buf, int& Layer, int& Channels, int& Samplerate, int& Bitrate);
	inline float ConvertSample(mad_fixed_t Sample);
};
